# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from mx import DateTime
import time
from osv import fields, osv

class stock_inventory(osv.osv):
    _inherit = "stock.inventory"
    
    #FIXME
    def action_done(self, cr, uid, ids, context=None):
        
        for inv in self.browse(cr, uid, ids):
            move_ids = []
            move_line = []
            for line in inv.inventory_line_id:
                pid = line.product_id.id
                price = line.product_id.standard_price or 0.0
                amount = self.pool.get('stock.location')._product_get(cr, uid, line.location_id.id, [pid], {'uom': line.product_uom.id})[pid]
                change = line.product_qty - amount
                if change:
                    location_id = line.product_id.product_tmpl_id.property_stock_inventory.id
                    value = {
                        'name': 'INV:' + str(line.inventory_id.id) + ':' + line.inventory_id.name,
                        'product_id': line.product_id.id,
                        'product_uom': line.product_uom.id,
                        'product_second_uom': line.product_second_uom.id,
                        'product_second_uom_qty': line.product_second_uom_qty,
                        'date': inv.date,
                        'date_planned': inv.date,
                        'state': 'assigned'
                    }
                    if change > 0:
                        value.update( {
                            'product_qty': change,
                            'product_second_uom': line.product_second_uom.id,
                            'product_second_uom_qty': line.product_second_uom_qty,
                            'location_id': location_id,
                            'location_dest_id': line.location_id.id,
                        })
                    else:
                        value.update( {
                            'product_qty': -change,
                            'product_second_uom': line.product_second_uom.id,
                            'product_second_uom_qty': line.product_second_uom_qty,
                            'location_id': line.location_id.id,
                            'location_dest_id': location_id,
                        })
                    move_ids.append(self.pool.get('stock.move').create(cr, uid, value))
            if len(move_ids):
                self.pool.get('stock.move').action_done(cr, uid, move_ids,
                        context=context)
            self.write(cr, uid, [inv.id], {'state': 'done', 'date_done': time.strftime('%Y-%m-%d %H:%M:%S'), 'move_ids': [(6, 0, move_ids)]})
        return True
    
stock_inventory()

class stock_inventory_line(osv.osv):
    _inherit = "stock.inventory.line"
    _columns = {
              'product_second_uom_qty': fields.float('Quantity (Second UoM)', digits=(16, 2)),
              'product_second_uom': fields.many2one('product.uom', 'Product Second UoM'),
              'related_use_second_uom': fields.related('product_id', 'use_second_uom', type='boolean', string='Use Second UOM'),
              }
    _defaults = {
                'product_second_uom_qty': lambda *a: 1
                }

    def on_change_product_id(self, cr, uid, ids, location_id, product, uom=False):

        result = super(stock_inventory_line, self).on_change_product_id(cr, uid, ids, location_id, product, uom=False)

        if product:
            product_obj = self.pool.get('product.product').browse(cr, uid, product)
            if product_obj.use_second_uom:
                    result['value'].update({'product_second_uom': product_obj.second_uom_id.id, 'related_use_second_uom': product_obj.use_second_uom})
            else:
                    result['value'].update({'product_second_uom_qty': False, 'product_second_uom': False, 'related_use_second_uom': False})
        else:
                aux = {'value': {'product_second_uom_qty': False, 'product_second_uom': False, 'related_use_second_uom': False}}
                result.update( aux )
        return result

stock_inventory_line()

class stock_move(osv.osv):
    _inherit = "stock.move"
    _columns = {
              'product_second_uom_qty': fields.float('Quantity (Second UoM)', digits=(16, 2)),
              'product_second_uom': fields.many2one('product.uom', 'Product Second UoM'),
              'related_use_second_uom': fields.related('product_id', 'use_second_uom', type='boolean', string='Use Second UOM'),
              }
    _defaults = {
                'product_second_uom_qty': lambda *a: 0
                }

    def onchange_product_id(self, cr, uid, ids, prod_id=False, loc_id=False, loc_dest_id=False):

        result = super(stock_move, self).onchange_product_id(cr, uid, ids, prod_id, loc_id, loc_dest_id)

        if not prod_id:
            return result.update({'product_second_uom': '', 'related_use_second_uom': False})

        product_obj = self.pool.get('product.product').browse(cr, uid, prod_id)
        if product_obj.use_second_uom:
            result['value'].update({'product_second_uom': product_obj.second_uom_id.id, 'related_use_second_uom': product_obj.use_second_uom})
        else:
            result['value'].update({'product_second_uom_qty': False, 'product_second_uom': False, 'related_use_second_uom': False})
        return result

stock_move()

class stock_location(osv.osv):
    _inherit = "stock.location"
    
    def _product_second_qty_available(self, cr, uid, ids, field_names, arg, context={}):
        res = {}
        for id in ids:
            res[id] = {}.fromkeys(field_names, 0.0)
        if ('product_id' not in context) or not ids:
            return res
        
        for loc in ids:
            context['location'] = [loc]
            prod = self.pool.get('product.product').browse(cr, uid, context['product_id'], context)
            if 'second_stock_real' in field_names:
                res[loc]['second_stock_real'] = prod.second_qty_available
            if 'second_stock_virtual' in field_names:
                res[loc]['second_stock_virtual'] = prod.second_virtual_available
        return res
    
    _columns = {
                #'uo_id': fields.many2one('product.uom', 'UOM', readonly=True, select=True),
                'second_stock_real': fields.function(_product_second_qty_available, method=True, type='float', string='Second Real Stock', multi="stock"),
                'second_stock_virtual': fields.function(_product_second_qty_available, method=True, type='float', string='Second Virtual Stock', multi="stock"),
                #'suo_id': fields.many2one('product.uom', 'Second UOM', readonly=True, select=True),
                }
        
stock_location()